@@ -18,7 +18,7 @@
 SignalTransition::SignalTransition(QState *parent)
     : QSignalTransition(this, SIGNAL(invokeYourself()), parent), m_complete(false), m_signalExpression(nullptr)
 {
-    connect(this, &SignalTransition::signalChanged, [this](){ m_signal.notify(); });
+    connect(this, &SignalTransition::signalChanged, this, [this](){ m_signal.notify(); });
 }
 
 bool SignalTransition::eventTest(QEvent *event)
@@ -89,10 +89,9 @@ const QJSValue& SignalTransition::signal()
 
 void SignalTransition::setSignal(const QJSValue &signal)
 {
-    if (m_signal.value().strictlyEquals(signal)) {
-        m_signal.removeBindingUnlessInWrapper();
+    m_signal.removeBindingUnlessInWrapper();
+    if (m_signal.valueBypassingBindings().strictlyEquals(signal))
         return;
-    }
 
     QV4::ExecutionEngine *jsEngine = QQmlEngine::contextForObject(this)->engine()->handle();
     QV4::Scope scope(jsEngine);
@@ -100,7 +99,7 @@ void SignalTransition::setSignal(const QJSValue &signal)
     QObject *sender;
     QMetaMethod signalMethod;
 
-    m_signal = signal;
+    m_signal.setValueBypassingBindings(signal);
     QV4::ScopedValue value(scope, QJSValuePrivate::asReturnedValue(&signal));
 
     // Did we get the "slot" that can be used to invoke the signal?
@@ -167,7 +166,11 @@ void SignalTransition::connectTriggered()
     QV4::Scope scope(jsEngine);
     QV4::Scoped<QV4::QObjectMethod> qobjectSignal(
                 scope, QJSValuePrivate::asReturnedValue(&m_signal.value()));
-    Q_ASSERT(qobjectSignal);
+    if (!qobjectSignal) {
+        m_signalExpression.adopt(nullptr);
+        return;
+    }
+
     QMetaMethod metaMethod = target->metaObject()->method(qobjectSignal->methodIndex());
     int signalIndex = QMetaObjectPrivate::signalIndex(metaMethod);
 
@@ -183,7 +186,9 @@ void SignalTransition::connectTriggered()
     }
 }
 
-void SignalTransitionParser::verifyBindings(const QQmlRefPointer<QV4::ExecutableCompilationUnit> &compilationUnit, const QList<const QV4::CompiledData::Binding *> &props)
+void SignalTransitionParser::verifyBindings(
+        const QQmlRefPointer<QV4::CompiledData::CompilationUnit> &compilationUnit,
+        const QList<const QV4::CompiledData::Binding *> &props)
 {
     for (int ii = 0; ii < props.size(); ++ii) {
         const QV4::CompiledData::Binding *binding = props.at(ii);
