@@ -5,10 +5,11 @@
 #define MOC_H
 
 // -- QtScxml
+#include <QtCore/qjsonarray.h>
+#include <QtCore/qjsondocument.h>
 #include <QtCore/qmap.h>
 #include <QtCore/qpair.h>
-#include <QtCore/qjsondocument.h>
-#include <QtCore/qjsonarray.h>
+#include <QtCore/qtmocconstants.h>
 // -- QtScxml
 
 #include <private/qtools_p.h>
@@ -17,6 +18,15 @@ QT_BEGIN_NAMESPACE
 
 struct QMetaObject;
 
+enum class TypeTag : uchar {
+    None,
+    HasStruct = 0x01,
+    HasClass = 0x02,
+    HasEnum = 0x04,
+};
+Q_DECLARE_FLAGS(TypeTags, TypeTag)
+Q_DECLARE_OPERATORS_FOR_FLAGS(TypeTags)
+
 struct Type
 {
     enum ReferenceType { NoReference, Reference, RValueReference, Pointer };
@@ -30,6 +40,7 @@ struct Type
     QByteArray rawName;
     uint isVolatile : 1;
     uint isScoped : 1;
+    TypeTags typeTag;
 #if 0 // -- QtScxml
     Token firstToken;
 #endif // -- QtScxml
@@ -44,8 +55,7 @@ struct EnumDef
     QByteArray enumName;
     QByteArray type;
     QList<QByteArray> values;
-    bool isEnumClass; // c++11 enum class
-    EnumDef() : isEnumClass(false) {}
+    QFlags<QtMocConstants::EnumFlags> flags = {};
     QJsonObject toJson(const ClassDef &cdef) const;
     QByteArray qualifiedType(const ClassDef *cdef) const;
 };
@@ -96,7 +106,7 @@ struct FunctionDef
     bool isAbstract = false;
     bool isRawSlot = false;
 
-    QJsonObject toJson() const;
+    QJsonObject toJson(int index) const;
     static void accessToJson(QJsonObject *obj, Access acs);
 
 // -- QtScxml
@@ -122,6 +132,7 @@ struct PropertyDef
     enum Specification  { ValueSpec, ReferenceSpec, PointerSpec };
     Specification gspec = ValueSpec;
     int revision = 0;
+    TypeTags typeTag;
     bool constant = false;
     bool final = false;
     bool required = false;
@@ -158,7 +169,7 @@ struct BaseDef {
     QByteArray classname;
     QByteArray qualified;
     QList<ClassInfoDef> classInfoList;
-    QMap<QByteArray, bool> enumDeclarations;
+    QMap<QByteArray, QFlags<QtMocConstants::EnumFlags>> enumDeclarations;
     QList<EnumDef> enumList;
     QMap<QByteArray, QByteArray> flagAliases;
     qsizetype begin = 0;
@@ -272,7 +283,7 @@ public:
     void createPropertyDef(PropertyDef &def, int propertyIndex, PropertyMode mode);
 
     void parsePropertyAttributes(PropertyDef &propDef);
-    void parseEnumOrFlag(BaseDef *def, bool isFlag);
+    void parseEnumOrFlag(BaseDef *def, QtMocConstants::EnumFlags flags);
     void parseFlag(BaseDef *def);
     enum class EncounteredQmlMacro {Yes, No};
     EncounteredQmlMacro parseClassInfo(BaseDef *def);
